useDynLib(relsurv, .registration=TRUE)
importFrom(Rcpp, evalCpp)
import(survival, splines)
importFrom("graphics", "abline", "axis", "box", "legend", "lines",
             "par", "plot", "points", "polygon")
importFrom("stats", "approx", "binomial", "coef", "delete.response",
             "glm", "glm.control", "median", "model.extract",
             "model.frame", "model.matrix", "pchisq", "pnorm", "poisson",
             "printCoefmat", "qchisq", "resid", "symnum", "terms","as.formula",
           "model.offset", "predict", "aggregate")
importFrom("utils", "getFromNamespace", "read.table")
export("epa","invtime","joinrate",
	"rs.br","rs.diff","rs.surv",
	"rs.zph","rsadd","rsmul","rstrans",
	"residuals.rsadd",
	"survsplit","transrate","transrate.hld",
	"transrate.hmd","rs.surv.rsadd","survfit.rsadd","cmp.rel","nessie", 'years', 'plot_f', 'plot_years',
	'predict.rsadd', 'predict.aalen.model', 'rsaalen', 'survaalen', 'popsurv')
S3method(residuals, rsadd)
S3method(plot, rs.br)
S3method(plot, cmp.rel)
S3method(print, cmp.rel)
S3method(plot, rs.zph)
S3method(print, rsadd)
S3method(summary, rsadd)
S3method(summary, cmp.rel)
S3method(print, summary.rsadd)
S3method(print, rsdiff)
S3method(print, rs.br)
S3method(survfit, rsadd)
S3method(predict, rsadd)
S3method(predict, aalen.model)
export(expprep2)
