# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

Yt <- function(data, times) {
    .Call(`_relsurv_Yt`, data, times)
}

dNt <- function(data, times) {
    .Call(`_relsurv_dNt`, data, times)
}

prepareX <- function(Yt, xt) {
    .Call(`_relsurv_prepareX`, Yt, xt)
}

fitOLS <- function(mX, dNt, Yt) {
    .Call(`_relsurv_fitOLS`, mX, dNt, Yt)
}

fitOLS2 <- function(mX, dNt, Yt) {
    .Call(`_relsurv_fitOLS2`, mX, dNt, Yt)
}

fitOLSconst <- function(mX, mZ, dNt, Yt) {
    .Call(`_relsurv_fitOLSconst`, mX, mZ, dNt, Yt)
}

rcpp_unlist <- function(listObject) {
    .Call(`_relsurv_rcpp_unlist`, listObject)
}

build_array3 <- function(x, dimensions) {
    .Call(`_relsurv_build_array3`, x, dimensions)
}

