\name{slopop}
\alias{slopop}
\docType{data}
\title{Census Data Set for the Slovene Population}
\description{
  Census data set for the Slovene population.
}
\usage{data(slopop)}

\examples{
data(slopop)
}
\keyword{datasets}
