\name{ebmt1wide}
\alias{ebmt1wide}
\docType{data}
\title{EBMT Data Taken From Package mstate}
\description{
  The ebmt1 dataset taken from package mstate transformed in long format.
}
\usage{data(ebmt1wide)}


\keyword{datasets}
